import os
from gi.repository import Gio

from .result_item import ResultItem
from .tools import sizeof_fmt


ALLOWED_MIME_TYPES = {"image/jpeg", "image/png", "image/webp", "image/svg+xml"}


class ResultItemManager:
    def __init__(self, settings_manager):
        self.settings = settings_manager

    def build(self, file) -> ResultItem:
        result_item = ResultItem()

        if not file.query_exists():
            result_item.set_error(_("This file doesn't exist."))
            return result_item

        # Get file info
        file_info = file.query_info(
            "standard::display-name,standard::size,standard::content-type,xattr::document-portal.host-path",
            Gio.FileQueryInfoFlags.NONE,
        )

        # Get path by checking host path
        host_path = file_info.get_attribute_string("xattr::document-portal.host-path")
        result_item.filename = host_path if host_path else file.get_path()

        # Display name
        display_name = file_info.get_display_name()
        result_item.name = display_name if display_name else file.get_basename()

        # Check format
        result_item.size = file_info.get_size()
        result_item.mime_type = file_info.get_content_type()
        if result_item.mime_type not in ALLOWED_MIME_TYPES or result_item.size <= 0:
            result_item.set_error(_("Format of this file is not supported."))
            return result_item

        result_item.subtitle_label = sizeof_fmt(result_item.size)

        # New path
        result_item.new_filename = self.create_new_filename(result_item.filename)

        # Tmp path
        base_dir, fname = os.path.split(result_item.new_filename)
        result_item.tmp_filename = os.path.join(base_dir, f".{fname}.tmp")

        return result_item

    def create_new_filename(self, path):
        new_filename = path
        basename = os.path.basename(path)
        splitext = os.path.splitext(basename)
        parent = path.replace(basename, "")
        stem = splitext[0]
        extension = splitext[1]
        suffix_prefix = self.settings.suffix_prefix

        # Use new file or not
        if self.settings.new_file:
            if self.settings.naming_mode == 0:  # Suffix selected
                new_filename = f"{parent}/{stem}{suffix_prefix}{extension}"
            else:  # Prefix selected
                new_filename = f"{parent}/{suffix_prefix}{stem}{extension}"

        return new_filename



from shlex import quote

from ..compressor import Compressor


class SVGCompressor(Compressor):
    @classmethod
    def get_file_type(cls) -> str:
        return "svg"

    def build_command(self, result_item) -> str:
        command = f"scour -i {quote(result_item.filename)} -o {quote(result_item.tmp_filename)}"

        if self.settings.svg_maximum_level:
            command += " --enable-viewboxing --enable-id-stripping"
            command += " --enable-comment-stripping --shorten-ids --indent=none"

        return command



import os
import threading
from concurrent.futures import ThreadPoolExecutor
from gi.repository import GLib


class CompressionManager:
    def __init__(self, settings_manager):
        self.settings = settings_manager
        self.compressors = {}

    def mime_type_to_compressor_type(self, mime_type: str) -> str:
        if mime_type == "image/jpeg":
            return "jpeg"
        elif mime_type == "image/png":
            return "png"
        elif mime_type == "image/webp":
            return "webp"
        elif mime_type == "image/svg+xml":
            return "svg"

    def register_compressor(self, ConcreteCompressor):
        file_type = ConcreteCompressor.get_file_type()
        if file_type not in self.compressors:
            self.compressors[file_type] = ConcreteCompressor(self.settings)

    def compress(self, result_items, c_update_result_item, c_enable_compression):
        threading.Thread(
            target=self._compress,
            args=(result_items, c_update_result_item, c_enable_compression),
            daemon=True,
        ).start()

    def _compress(self, result_items, c_update_result_item, c_enable_compression):
        cpu_count = os.cpu_count()
        executor = ThreadPoolExecutor(max_workers=cpu_count)
        futures = []
        for result_item in result_items:
            compressor_type = self.mime_type_to_compressor_type(result_item.mime_type)
            future = executor.submit(
                self.compressors[compressor_type].run, result_item, c_update_result_item
            )
            futures.append(future)

        for future in futures:
            future.result()
        GLib.idle_add(c_enable_compression, True)



from gi.repository import Gtk, Adw, GObject

from .result_item import ResultItem
from .tools import create_image_from_file

CURTAIL_PATH = "/com/github/huluti/Curtail/"


@Gtk.Template(resource_path=CURTAIL_PATH + "ui/result-item-row.ui")
class CurtailResultItemRow(Adw.ActionRow):
    __gtype_name__ = "CurtailResultItemRow"

    result_item = GObject.Property(type=ResultItem)

    skipped_info_button = Gtk.Template.Child()
    error_info_button = Gtk.Template.Child()
    error_popover_label = Gtk.Template.Child()
    savings_label = Gtk.Template.Child()
    spinner = Gtk.Template.Child()
    error_image = Gtk.Template.Child()

    def __init__(self, result_item: ResultItem, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.result_item = result_item

        self.set_title(result_item.name)
        self.set_tooltip_text(result_item.new_filename)
        self.set_subtitle(result_item.subtitle_label)

        if len(result_item.new_filename) > 0:
            image = create_image_from_file(result_item.filename, 48, 48)
            if image:
                self.add_prefix(image)

        result_item.bind_property(
            "savings", self.savings_label, "label", GObject.BindingFlags.DEFAULT
        )
        result_item.bind_property(
            "subtitle_label", self, "subtitle", GObject.BindingFlags.DEFAULT
        )
        result_item.bind_property(
            "running", self.spinner, "visible", GObject.BindingFlags.DEFAULT
        )
        result_item.bind_property(
            "skipped", self.skipped_info_button, "visible", GObject.BindingFlags.DEFAULT
        )
        result_item.bind_property(
            "error", self.error_image, "visible", GObject.BindingFlags.DEFAULT
        )
        result_item.bind_property(
            "error_details", self.error_info_button, "visible", GObject.BindingFlags.DEFAULT
        )
        result_item.bind_property(
            "error_details_message",
            self.error_popover_label,
            "label",
            GObject.BindingFlags.DEFAULT
        )



from gi.repository import Gio, GObject


class ResultItem(GObject.Object):
    file = GObject.Property(type=Gio.File)
    mime_type = GObject.Property(type=str)
    name = GObject.Property(type=str)
    filename = GObject.Property(type=str)
    new_filename = GObject.Property(type=str)
    tmp_filename = GObject.Property(type=str)
    size = GObject.Property(type=int, default=0)
    new_size = GObject.Property(type=int, default=0)
    subtitle_label = GObject.Property(type=str, default="")
    savings = GObject.Property(type=str, default="")
    running = GObject.Property(type=bool, default=True)
    skipped = GObject.Property(type=bool, default=False)
    error = GObject.Property(type=bool, default=False)
    error_message = GObject.Property(type=str, default="")
    error_details = GObject.Property(type=bool, default=False)
    error_details_message = GObject.Property(type=str, default="")

    def __repr__(self):
        return str(self.name)

    def set_error(self, error: str) -> None:
        self.error = True
        self.error_message = error



import logging
import subprocess
import html
from abc import ABC, abstractmethod
from typing import Callable

from gi.repository import Gio, GLib

from .result_item import ResultItem


class Compressor(ABC):
    def __init__(self, settings):
        super().__init__()
        self.settings = settings

    @classmethod
    @abstractmethod
    def get_file_type(cls) -> str:
        return ""

    @abstractmethod
    def build_command(cls, result_item: ResultItem) -> str:
        return ""

    def run(self, result_item: ResultItem, c_update_result_item: Callable) -> None:
        command = self.build_command(result_item)
        try:
            output = subprocess.run(
                command,
                capture_output=True,
                check=True,
                shell=True,
                timeout=self.settings.compression_timeout,
            )
        except subprocess.TimeoutExpired as err:
            logging.error(str(err))
            result_item.error_message = _(
                f"Compression has reached the configured timeout of {self.settings.compression_timeout} seconds."
            )
            result_item.error = True
        except Exception as err:
            result_item.error_message = _("An unknown error has occurred.")
            result_item.error_details_message = html.escape(str(err))
            logging.error(result_item.error_details_message)
            result_item.error = True
            result_item.error_details = True

        if result_item.error:
            GLib.idle_add(c_update_result_item, result_item)
            return

        new_file = Gio.File.new_for_path(result_item.tmp_filename)
        if new_file.query_exists():
            new_file_info = new_file.query_info(
                "standard::size", Gio.FileQueryInfoFlags.NONE
            )
            result_item.new_size = new_file_info.get_size()

            if result_item.new_size >= result_item.size:
                # Output is larger (or equal) than input
                # Don't use compressed temp file
                result_item.skipped = True
            else:
                # Output is smaller than input
                # Copy the compressed temp file
                final_path = (
                    result_item.new_filename
                    if self.settings.new_file
                    else result_item.filename
                )
                source = Gio.File.new_for_path(result_item.tmp_filename)
                dest = Gio.File.new_for_path(final_path)
                source.copy(
                    dest, Gio.FileCopyFlags.OVERWRITE | Gio.FileCopyFlags.ALL_METADATA
                )

            # Remove the temp file
            new_file.delete()
        else:
            logging.error(str(output))
            result_item.error_message = _("Can't find the compressed file")
            result_item.error = True

        GLib.idle_add(c_update_result_item, result_item)



from gi.repository import Gtk, Adw

from .settings_manager import SettingsManager


UI_PATH = "/com/github/huluti/Curtail/ui/"


@Gtk.Template(resource_path=UI_PATH + "preferences.ui")
class CurtailPrefsDialog(Adw.PreferencesDialog):
    __gtype_name__ = "CurtailPrefsDialog"

    toggle_recursive = Gtk.Template.Child()
    toggle_metadata = Gtk.Template.Child()
    toggle_file_attributes = Gtk.Template.Child()
    toggle_new_file = Gtk.Template.Child()
    toggle_naming_mode = Gtk.Template.Child()
    entry_suffix_prefix = Gtk.Template.Child()
    spin_timeout = Gtk.Template.Child()
    spin_png_lossy_level = Gtk.Template.Child()
    spin_png_lossless_level = Gtk.Template.Child()
    spin_webp_lossless_level = Gtk.Template.Child()
    spin_jpg_lossy_level = Gtk.Template.Child()
    spin_webp_lossy_level = Gtk.Template.Child()
    toggle_jpg_progressive = Gtk.Template.Child()
    toggle_svg_maximum_level = Gtk.Template.Child()

    def __init__(self, parent, **kwargs):
        super().__init__(**kwargs)

        self.settings = SettingsManager()

        self.parent = parent
        self.build_ui()

    def build_ui(self):
        # Compression settings

        # Recursive
        self.toggle_recursive.set_active(self.settings.recursive)
        self.toggle_recursive.connect(
            "notify::active", self.on_bool_changed, "recursive"
        )

        # Keep metadata
        self.toggle_metadata.set_active(self.settings.metadata)
        self.toggle_metadata.connect("notify::active", self.on_bool_changed, "metadata")

        # Preserve file attributes
        self.toggle_file_attributes.set_active(self.settings.file_attributes)
        self.toggle_file_attributes.connect(
            "notify::active", self.on_bool_changed, "file-attributes"
        )

        # Use new file
        self.toggle_new_file.set_active(self.settings.new_file)
        self.toggle_new_file.connect("notify::active", self.on_bool_changed, "new-file")

        # Naming mode
        self.toggle_naming_mode.set_sensitive(self.settings.new_file)
        self.toggle_naming_mode.set_selected(self.settings.naming_mode)
        self.toggle_naming_mode.connect(
            "notify::selected-item", self.on_selected_item, "naming-mode"
        )

        # Prefix-Suffix
        self.entry_suffix_prefix.set_sensitive(self.settings.new_file)
        self.entry_suffix_prefix.set_text(self.settings.suffix_prefix)
        self.entry_suffix_prefix.connect(
            "changed", self.on_string_changed, "suffix-prefix"
        )

        # Compression Timeout
        self.spin_timeout.set_value(self.settings.compression_timeout)
        self.spin_timeout.connect(
            "notify::value", self.on_int_changed, "compression-timeout"
        )

        # PNG Lossless Compression Level
        self.spin_png_lossless_level.set_value(self.settings.png_lossless_level)
        self.spin_png_lossless_level.connect(
            "notify::value", self.on_int_changed, "png-lossless-level"
        )

        # PNG Lossy Compression Level
        self.spin_png_lossy_level.set_value(self.settings.png_lossy_level)
        self.spin_png_lossy_level.connect(
            "notify::value", self.on_int_changed, "png-lossy-level"
        )

        # WebP Lossless Compression Level
        self.spin_webp_lossless_level.set_value(self.settings.webp_lossless_level)
        self.spin_webp_lossless_level.connect(
            "notify::value", self.on_int_changed, "webp-lossless-level"
        )

        # WebP Lossy Compression Level
        self.spin_webp_lossy_level.set_value(self.settings.webp_lossy_level)
        self.spin_webp_lossy_level.connect(
            "notify::value", self.on_int_changed, "webp-lossy-level"
        )

        # JPG Lossy Compression Level
        self.spin_jpg_lossy_level.set_value(self.settings.jpg_lossy_level)
        self.spin_jpg_lossy_level.connect(
            "notify::value", self.on_int_changed, "jpg-lossy-level"
        )

        # Progressively Encode JPG
        self.toggle_jpg_progressive.set_active(self.settings.jpg_progressive)
        self.toggle_jpg_progressive.connect(
            "notify::active", self.on_bool_changed, "jpg-progressive"
        )

        # Maxium SVG compression
        self.toggle_svg_maximum_level.set_active(self.settings.svg_maximum_level)
        self.toggle_svg_maximum_level.connect(
            "notify::active", self.on_bool_changed, "svg-maximum-level"
        )

    def on_bool_changed(self, switch, state, key):
        self.settings.set_boolean(key, switch.get_active())
        # Additional actions
        if key == "new-file":
            new_file = self.settings.new_file
            self.parent.set_saving_subtitle(new_file)
            self.parent.show_warning_banner(not new_file)
            self.toggle_naming_mode.set_sensitive(new_file)
            self.entry_suffix_prefix.set_sensitive(new_file)

    def on_selected_item(self, combo, _, key):
        self.settings.set_int(key, combo.get_selected())
        if key == "naming-mode":
            if not self.settings.naming_mode:
                self.settings.reset("naming-mode")
            self.parent.set_saving_subtitle()

    def on_string_changed(self, entry, key):
        self.settings.set_string(key, entry.get_text())
        if key == "suffix-prefix":
            if not self.settings.suffix_prefix:
                self.settings.reset("suffix-prefix")
            self.parent.set_saving_subtitle()

    def on_int_changed(self, spin, _, key):
        self.settings.set_int(key, spin.get_value())



import os
import subprocess
from gi.repository import Gtk, Gdk, Gio, GLib, Adw

from .compressors.png_compressor import PNGCompressor
from .compressors.jpeg_compressor import JPEGCompressor
from .compressors.webp_compressor import WEBPCompressor
from .compressors.svg_compressor import SVGCompressor
from .compression_manager import CompressionManager
from .settings_manager import SettingsManager
from .result_item import ResultItem
from .result_item_manager import ResultItemManager
from .result_item_row import CurtailResultItemRow
from .preferences import CurtailPrefsDialog
from .tools import (
    add_filechooser_filters,
    sizeof_fmt,
    debug_infos,
    get_image_files_from_folder,
    get_image_files_from_folder_recursive,
)

CURTAIL_PATH = "/com/github/huluti/Curtail/"


@Gtk.Template(resource_path=CURTAIL_PATH + "ui/window.ui")
class CurtailWindow(Adw.ApplicationWindow):
    __gtype_name__ = "CurtailWindow"

    prefs_dialog = None
    apply_window = None

    headerbar = Gtk.Template.Child()
    window_title = Gtk.Template.Child()
    toast_overlay = Gtk.Template.Child()
    filechooser_button_headerbar = Gtk.Template.Child()
    clear_button_headerbar = Gtk.Template.Child()
    warning_banner = Gtk.Template.Child()
    mainbox = Gtk.Template.Child()
    loadingbox = Gtk.Template.Child()
    homebox = Gtk.Template.Child()
    resultbox = Gtk.Template.Child()
    listbox = Gtk.Template.Child()
    toggle_lossy = Gtk.Template.Child()

    results_model = Gio.ListStore.new(ResultItem)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.set_default_icon_name("com.github.huluti.Curtail")
        self.app = kwargs["application"]

        self.settings = SettingsManager()

        self.build_ui()
        self.create_actions()
        self.show_view("home")

        self.manager = CompressionManager(self.settings)
        self.manager.register_compressor(PNGCompressor)
        self.manager.register_compressor(JPEGCompressor)
        self.manager.register_compressor(WEBPCompressor)
        self.manager.register_compressor(SVGCompressor)

        self.result_item_manager = ResultItemManager(self.settings)

    def build_ui(self):
        # Set icons
        icon_theme = Gtk.IconTheme.get_for_display(Gdk.Display.get_default())
        icon_theme.add_resource_path(CURTAIL_PATH + "icons/")

        # Saving subtitle
        self.set_saving_subtitle()

        # Warning banner
        self.show_warning_banner()

        # Mainbox - drag&drop
        drop_target_main = Gtk.DropTarget.new(
            type=Gdk.FileList, actions=Gdk.DragAction.COPY
        )
        drop_target_main.connect("drop", self.on_dnd_drop)
        self.mainbox.add_controller(drop_target_main)

        # Lossy toggle
        self.toggle_lossy.set_active(self.settings.lossy)
        self.toggle_lossy.connect("notify::active", self.on_lossy_changed)

        # Results
        self.listbox.bind_model(
                self.results_model, lambda result_item : CurtailResultItemRow(result_item)
        )

        # Right click on results
        gesture = Gtk.GestureClick.new()
        gesture.set_button(Gdk.BUTTON_SECONDARY)
        gesture.connect("pressed", self.on_results_right_click, self.listbox)
        self.listbox.add_controller(gesture)

    def create_simple_action(self, action_name, callback, shortcut=None):
        action = Gio.SimpleAction.new(action_name, None)
        action.connect("activate", callback)
        self.add_action(action)
        if shortcut is not None:
            self.app.set_accels_for_action("win." + action_name, [shortcut])

    def create_actions(self):
        self.create_simple_action("select-file", self.on_select, "<Primary>o")
        self.create_simple_action("clear-results", self.clear_results)
        self.create_simple_action("banner-change-mode", self.banner_change_mode)
        self.create_simple_action("preferences", self.on_preferences, "<Primary>comma")
        self.create_simple_action("about", self.on_about)
        self.create_simple_action("quit", self.on_quit, "<Primary>q")
        self.create_simple_action("convert-dir", self.on_select_folder, "<Primary>d")

    def enable_compression(self, enable):
        self.filechooser_button_headerbar.set_sensitive(enable)
        self.clear_button_headerbar.set_sensitive(enable)

    def show_view(self, view):
        if view == "home":
            self.homebox.set_visible(True)
            self.loadingbox.set_visible(False)
            self.resultbox.set_visible(False)
            self.clear_button_headerbar.set_visible(False)
        elif view == "loading":
            self.homebox.set_visible(False)
            self.loadingbox.set_visible(True)
            self.resultbox.set_visible(False)
            self.clear_button_headerbar.set_visible(False)
        elif view == "results":
            self.homebox.set_visible(False)
            self.loadingbox.set_visible(False)
            self.resultbox.set_visible(True)
            self.clear_button_headerbar.set_visible(True)

    def clear_results(self, *args):
        self.show_view("home")
        self.results_model.remove_all()

    def update_result_item(self, result_item):
        result_item.running = False
        if result_item.error:
            result_item.subtitle_label = result_item.error_message
        elif result_item.skipped:
            result_item.savings = _("Skipped")
        else:
            result_item.savings = (
                str(round(100 - (result_item.new_size * 100 / result_item.size), 2))
                + "%"
            )
            result_item.subtitle_label += " → " + sizeof_fmt(result_item.new_size)

    def on_results_right_click(self, gesture, button, x, y, user_data):
        row = self.listbox.get_row_at_y(y)
        filename = row.get_tooltip_text()
        if not filename or not os.path.exists(filename):
            return

        popover = Gtk.PopoverMenu()

        # Create actions for the popover
        action_group = Gio.SimpleActionGroup.new()

        open_image_action = Gio.SimpleAction.new("open-image", None)
        open_image_action.connect("activate", self.on_open_image_action, filename)
        action_group.add_action(open_image_action)

        open_folder_action = Gio.SimpleAction.new("open-folder", None)
        open_folder_action.connect("activate", self.on_open_folder_action, filename)
        action_group.add_action(open_folder_action)

        # Insert the actions into the window's action group
        self.insert_action_group("context-menu", action_group)

        # Define the structure of the popover menu
        menu_model = Gio.Menu.new()
        menu_model.append(_("Open Image"), "context-menu.open-image")
        menu_model.append(_("Show in Folder"), "context-menu.open-folder")

        # Set the menu model to the popover
        popover.set_menu_model(menu_model)
        popover.set_pointing_to(Gdk.Rectangle(x, y, 1, 1))
        popover.set_offset(x, y)
        popover.set_has_arrow(False)
        popover.set_parent(self.listbox)
        popover.popup()

    def on_open_image_action(self, action, param, filename):
        subprocess.run(["xdg-open", filename])

    def on_open_folder_action(self, action, param, filename):
        folder_path = os.path.dirname(filename)
        subprocess.run(["xdg-open", folder_path])

    def set_saving_subtitle(self, new_file=None):
        if new_file is None:
            new_file = self.settings.new_file
        if new_file:
            suffix_prefix = self.settings.suffix_prefix

            if self.settings.naming_mode == 0:
                label = _(f"Safe mode with “{suffix_prefix}” suffix")
            else:
                label = _(f"Safe mode with “{suffix_prefix}” prefix")
        else:
            label = _("Overwrite mode")
        self.window_title.set_subtitle(label)

    def show_warning_banner(self, show=None):
        if show is None:
            show = not self.settings.new_file

        self.warning_banner.set_revealed(show)

    def on_select(self, *args):
        dialog = Gtk.FileDialog(title=_("Browse Files"))
        add_filechooser_filters(dialog)

        def handle_response(dialog, result):
            try:
                files = dialog.open_multiple_finish(result)
            except GLib.Error as err:
                print("Could not open files: %s", err.message)
            else:
                self.show_view("loading")
                filenames = list()
                for file in files:
                    filenames.append(file.get_uri())
                self.compress_files(files)

        dialog.open_multiple(self, None, handle_response)

    def on_select_folder(self, *args):
        dialog = Gtk.FileDialog(title=_("Browse Directories"))

        def handle_response(dialog, result):
            def on_dir_dialog_response(warn_dialog, response):
                if response == "compress":
                    self.show_view("loading")
                    filenames = list()
                    for folder in folders:
                        filenames.append(folder)
                    self.compress_files(filenames)

            try:
                folders = dialog.select_multiple_folders_finish(result)
            except GLib.Error as err:
                print("Could not open files: %s", err.message)
            else:
                warning_message_dialog = self._create_warning_dialog()
                warning_message_dialog.connect("response", on_dir_dialog_response)
                warning_message_dialog.present(self)

        dialog.select_multiple_folders(self, None, handle_response)

    def _create_warning_dialog(self):
        dialog = None
        if self.settings.new_file:
            dialog = Adw.AlertDialog.new(
                _("Are you sure you want to compress images in these directories?"),
                _(
                    "All of the images in the directories selected and their "
                    "subdirectories will be compressed. The original images will not "
                    "be modified."
                ),
            )
        else:
            dialog = Adw.AlertDialog.new(
                _("Are you sure you want to compress images in these directories?"),
                _(
                    "All of the images in the directories selected and their "
                    "subdirectories will be compressed and overwritten!"
                ),
            )

        dialog.add_response("cancel", _("Cancel"))
        dialog.add_response("compress", _("Compress"))

        if self.settings.new_file:
            dialog.set_response_appearance("compress", Adw.ResponseAppearance.SUGGESTED)
        else:
            dialog.set_response_appearance(
                "compress", Adw.ResponseAppearance.DESTRUCTIVE
            )

        return dialog

    def on_dnd_drop(self, drop_target, value, x, y, user_data=None):
        self.show_view("loading")
        files = value.get_files()
        if not files:
            return

        self.compress_files(files)

    def handle_files(self, files):
        final_files = []
        for file in files:
            file_type = file.query_file_type(Gio.FileQueryInfoFlags.NONE)

            if file_type == Gio.FileType.DIRECTORY:
                if self.settings.recursive:
                    images = get_image_files_from_folder_recursive(file)
                else:
                    images = get_image_files_from_folder(file)
                final_files.extend(images)
            else:
                final_files.append(file)

        return final_files

    def compress_files(self, files):
        files = self.handle_files(files)
        if not files:
            self.toast_overlay.add_toast(Adw.Toast(title=_("No files found")))
            return

        result_items = []
        for file in files:
            result_item = self.result_item_manager.build(file)
            self.results_model.append(result_item)

            if result_item.error:
                GLib.idle_add(self.update_result_item, result_item)
            else:
                result_items.append(result_item)

        self.show_view("results")
        self.enable_compression(False)

        GLib.idle_add(
            self.manager.compress,
            result_items,
            self.update_result_item,
            self.enable_compression,
        )

    def on_lossy_changed(self, toggle, state):
        self.settings.lossy = toggle.get_active_name() == "lossy"

    def banner_change_mode(self, *args):
        self.settings.new_file = True
        self.show_warning_banner()
        self.set_saving_subtitle()

    def on_preferences(self, *args):
        if self.prefs_dialog is not None:
            self.prefs_dialog.force_close()
        self.prefs_dialog = CurtailPrefsDialog(self)
        self.prefs_dialog.present(self)

    def on_about(self, *args):
        about = Adw.AboutDialog(
            application_name="Curtail",
            application_icon="com.github.huluti.Curtail",
            developer_name="Hugo Posnic",
            license_type=Gtk.License.GPL_3_0,
            website="https://github.com/Huluti/Curtail",
            issue_url="https://github.com/Huluti/Curtail/issues/new",
            version="1.16.2",
            developers=["Hugo Posnic https://github.com/Huluti"],
            designers=[
                "Jakub Steiner https://github.com/jimmac",
                "Tobias Bernard https://github.com/bertob",
            ],
            translator_credits=_("translator-credits"),
            copyright="© Hugo Posnic",
        )
        about.add_credit_section(
            _("Contributors"),
            [
                "Steven Teskey",
                "Andrey Kozlovskiy",
                "Balló György",
                "olokelo",
                "Archisman Panigrahi",
                "Maximiliano",
                "ARAKHNID",
            ],
        )
        about.set_debug_info(debug_infos())
        about.present(self)

    def on_quit(self, *args):
        self.app.quit()



from shlex import quote

from ..compressor import Compressor


class PNGCompressor(Compressor):
    @classmethod
    def get_file_type(cls) -> str:
        return "png"

    def build_command(self, result_item) -> str:
        pngquant = "pngquant --quality=0-{} -f {} --output {}"
        oxipng = "oxipng -o {} -i 1 {} --out {}"

        if not self.settings.metadata:
            pngquant += " --strip"
            oxipng += " --strip safe"

        if self.settings.file_attributes:
            oxipng += " --preserve"

        if self.settings.lossy:  # lossy compression
            command = pngquant.format(
                self.settings.png_lossy_level,
                quote(result_item.filename),
                quote(result_item.tmp_filename),
            )
            command += " && "
            command += oxipng.format(
                self.settings.png_lossless_level,
                quote(result_item.tmp_filename),
                quote(result_item.tmp_filename),
            )
        else:  # lossless compression
            command = oxipng.format(
                self.settings.png_lossless_level,
                quote(result_item.filename),
                quote(result_item.tmp_filename),
            )
        return command



from shlex import quote

from ..compressor import Compressor


class JPEGCompressor(Compressor):
    @classmethod
    def get_file_type(cls) -> str:
        return "jpeg"

    def build_command(self, result_item) -> str:
        jpegoptim = "jpegoptim --max={} -o --stdout {} > {}"
        jpegoptim2 = "jpegoptim -o --stdout {} > {}"

        if self.settings.jpg_progressive:
            jpegoptim += " --all-progressive"
            jpegoptim2 += " --all-progressive"

        if not self.settings.metadata:
            jpegoptim += " --strip-all"
            jpegoptim2 += " --strip-all"

        if self.settings.file_attributes:
            jpegoptim += " --preserve --preserve-perms"
            jpegoptim2 += " --preserve --preserve-perms"

        if self.settings.lossy:  # lossy compression
            command = jpegoptim.format(
                self.settings.jpg_lossy_level,
                quote(result_item.filename),
                quote(result_item.tmp_filename),
            )
        else:  # lossless compression
            command = jpegoptim2.format(
                quote(result_item.filename), quote(result_item.tmp_filename)
            )
        return command



from gi.repository import Gio

SETTINGS_SCHEMA = "com.github.huluti.Curtail"


class SettingsManager:
    def __init__(self) -> None:
        self._settings = Gio.Settings.new(SETTINGS_SCHEMA)

    # Genereic setters
    def reset(self, key: str) -> None:
        self._settings.reset(key)

    def set_boolean(self, key: str, value: bool) -> None:
        self._settings.set_boolean(key, value)

    def set_int(self, key: str, value: int) -> None:
        self._settings.set_int(key, value)

    def set_string(self, key: str, value: str) -> None:
        self._settings.set_string(key, value)

    # Options
    @property
    def new_file(self) -> bool:
        return self._settings.get_boolean("new-file")

    @new_file.setter
    def new_file(self, value: bool) -> None:
        self._settings.set_boolean("new-file", value)

    @property
    def naming_mode(self) -> int:
        return self._settings.get_int("naming-mode")

    @naming_mode.setter
    def naming_mode(self, value: int) -> None:
        self._settings.set_int("naming-mode", value)

    @property
    def suffix_prefix(self) -> str:
        return self._settings.get_string("suffix-prefix")

    @suffix_prefix.setter
    def suffix_prefix(self, value: str) -> None:
        self._settings.set_string("suffix-prefix", value)

    @property
    def recursive(self) -> bool:
        return self._settings.get_boolean("recursive")

    @recursive.setter
    def recursive(self, value: bool) -> None:
        self._settings.set_boolean("recursive", value)

    @property
    def compression_timeout(self) -> int:
        return self._settings.get_int("compression-timeout")

    @compression_timeout.setter
    def compression_timeout(self, value: int) -> None:
        self._settings.set_int("compression-timeout", value)

    @property
    def lossy(self) -> bool:
        return self._settings.get_boolean("lossy")

    @lossy.setter
    def lossy(self, value: bool) -> None:
        self._settings.set_boolean("lossy", value)

    @property
    def metadata(self) -> bool:
        return self._settings.get_boolean("metadata")

    @metadata.setter
    def metadata(self, value: bool) -> None:
        self._settings.set_boolean("metadata", value)

    @property
    def file_attributes(self) -> bool:
        return self._settings.get_boolean("file-attributes")

    @file_attributes.setter
    def file_attributes(self, value: bool) -> None:
        self._settings.set_boolean("file-attributes", value)

    # PNG options
    @property
    def png_lossy_level(self) -> int:
        return self._settings.get_int("png-lossy-level")

    @png_lossy_level.setter
    def png_lossy_level(self, value: int) -> None:
        self._settings.set_int("png-lossy-level", value)

    @property
    def png_lossless_level(self) -> int:
        return self._settings.get_int("png-lossless-level")

    @png_lossless_level.setter
    def png_lossless_level(self, value: int) -> None:
        self._settings.set_int("png-lossless-level", value)

    # JPG options
    @property
    def jpg_lossy_level(self) -> int:
        return self._settings.get_int("jpg-lossy-level")

    @jpg_lossy_level.setter
    def jpg_lossy_level(self, value: int) -> None:
        self._settings.set_int("jpg-lossy-level", value)

    @property
    def jpg_progressive(self) -> bool:
        return self._settings.get_boolean("jpg-progressive")

    @jpg_progressive.setter
    def jpg_progressive(self, value: bool) -> None:
        self._settings.set_boolean("jpg-progressive", value)

    # WebP options
    @property
    def webp_lossless_level(self) -> int:
        return self._settings.get_int("webp-lossless-level")

    @webp_lossless_level.setter
    def webp_lossless_level(self, value: int) -> None:
        self._settings.set_int("webp-lossless-level", value)

    @property
    def webp_lossy_level(self) -> int:
        return self._settings.get_int("webp-lossy-level")

    @webp_lossy_level.setter
    def webp_lossy_level(self, value: int) -> None:
        self._settings.set_int("webp-lossy-level", value)

    # SVG options
    @property
    def svg_maximum_level(self) -> bool:
        return self._settings.get_boolean("svg-maximum-level")

    @svg_maximum_level.setter
    def svg_maximum_level(self, value: bool) -> None:
        self._settings.set_boolean("svg-maximum-level", value)



import sys
import gi

gi.require_version("Gtk", "4.0")
gi.require_version("Adw", "1")

from gi.repository import Gio, Adw  # noqa: E402

from .window import CurtailWindow  # noqa: E402


APP_ID = "com.github.huluti.Curtail"


class Application(Adw.Application):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.win: CurtailWindow | None = None

    def do_startup(self):
        Adw.Application.do_startup(self)

    def do_activate(self):
        if not self.win:
            self.win = CurtailWindow(application=self)
        self.win.present()

    def do_open(self, g_file_list, amount, ukwn):
        self.do_activate()
        self.win.compress_files(g_file_list)


def main(version):
    app = Application(application_id=APP_ID, flags=Gio.ApplicationFlags.HANDLES_OPEN)
    return app.run(sys.argv)



from shlex import quote

from ..compressor import Compressor
from ..result_item import ResultItem


class WEBPCompressor(Compressor):
    @classmethod
    def get_file_type(cls) -> str:
        return "webp"

    def build_command(self, result_item: ResultItem) -> str:
        command = f"cwebp {quote(result_item.filename)}"

        # cwebp doesn't preserve any metadata by default
        if self.settings.metadata:
            command += " -metadata all"

        if self.settings.lossy:
            quality = self.settings.webp_lossy_level
        else:
            command += " -lossless"
            quality = 100  # maximum cpu power for lossless

        # multithreaded, (lossless) compression mode, quality, output
        command += f" -mt -m {self.settings.webp_lossless_level}"
        command += f" -q {quality}"
        command += f" -o {quote(result_item.tmp_filename)}"

        return command



import re
import logging
import platform
import subprocess
from gi.repository import Gtk, GLib, Gio, GdkPixbuf


def sizeof_fmt(num):
    return GLib.format_size(num)


def add_filechooser_filters(dialog):
    all_images = Gtk.FileFilter()
    all_images.set_name(_("All images"))
    all_images.add_mime_type("image/jpeg")
    all_images.add_mime_type("image/png")
    all_images.add_mime_type("image/webp")
    all_images.add_mime_type("image/svg+xml")

    png_images = Gtk.FileFilter()
    png_images.set_name(_("PNG images"))
    png_images.add_mime_type("image/png")

    jpeg_images = Gtk.FileFilter()
    jpeg_images.set_name(_("JPEG images"))
    jpeg_images.add_mime_type("image/jpeg")

    webp_images = Gtk.FileFilter()
    webp_images.set_name(_("WebP images"))
    webp_images.add_mime_type("image/webp")

    svg_images = Gtk.FileFilter()
    svg_images.set_name(_("SVG images"))
    svg_images.add_mime_type("image/svg+xml")

    file_filters = Gio.ListStore.new(Gtk.FileFilter)
    file_filters.append(all_images)
    file_filters.append(png_images)
    file_filters.append(jpeg_images)
    file_filters.append(webp_images)
    file_filters.append(svg_images)

    dialog.set_filters(file_filters)


def create_image_from_file(filename, max_width, max_height):
    # Image preview
    try:
        pixbuf = GdkPixbuf.Pixbuf.new_from_file(filename)
    except Exception as err:
        logging.error(str(err))
        return None

    # Calculate new dimensions while preserving aspect ratio
    width = pixbuf.get_width()
    height = pixbuf.get_height()

    # If the image is wider than it is tall, scale it to fit the width
    if width > height:
        ratio = max_width / float(width)
        new_width = max_width
        new_height = int(height * ratio)
    # Otherwise, scale it to fit the height
    else:
        ratio = max_height / float(height)
        new_width = int(width * ratio)
        new_height = max_height

    scaled_pixbuf = pixbuf.scale_simple(
        new_width, new_height, GdkPixbuf.InterpType.BILINEAR
    )

    image = Gtk.Image.new_from_pixbuf(scaled_pixbuf)
    if new_width > new_height:
        image.set_pixel_size(new_width)
    else:
        image.set_pixel_size(new_height)

    return image


def get_image_files_from_folder(files):
    images = []
    enumerator = files.enumerate_children("standard::name", Gio.FileQueryInfoFlags.NONE)

    while file_info := enumerator.next_file():
        file_type = file_info.get_file_type()
        if file_type == Gio.FileType.DIRECTORY:
            continue

        image_file = enumerator.get_child(file_info)
        images.append(image_file)

    return images


def get_image_files_from_folder_recursive(files):
    images = []
    enumerator = files.enumerate_children("standard::name", Gio.FileQueryInfoFlags.NONE)

    while file_info := enumerator.next_file():
        file_type = file_info.get_file_type()
        image_file = enumerator.get_child(file_info)

        if file_type == Gio.FileType.DIRECTORY:
            images.extend(get_image_files_from_folder_recursive(image_file))
        else:
            images.append(image_file)

    return images


def debug_infos():
    python_version = platform.python_version()

    gtk_version = f"{Gtk.get_major_version()}.{Gtk.get_minor_version()}.{Gtk.get_micro_version()}"

    # Jpegoptim
    try:
        jpegoptim = subprocess.check_output(["jpegoptim", "--version"])
        jpegoptim = extract_version(jpegoptim.decode("utf-8"))
    except Exception:
        jpegoptim = _("Version not found")

    # Oxipng
    try:
        oxipng = subprocess.check_output(["oxipng", "--version"])
        oxipng = extract_version(oxipng.decode("utf-8"))
    except Exception:
        oxipng = _("Version not found")

    # pngquant
    try:
        pngquant = subprocess.check_output(["pngquant", "--version"])
        pngquant = extract_version(pngquant.decode("utf-8"))
    except Exception:
        pngquant = _("Version not found")

    # Libwebp
    try:
        libwebp = subprocess.check_output(["cwebp", "-version"])
        libwebp = extract_version(libwebp.decode("utf-8"))
    except Exception:
        libwebp = _("Version not found")

    # Scour
    try:
        scour = subprocess.check_output(["scour", "--version"])
        scour = extract_version(scour.decode("utf-8"))
    except Exception:
        scour = _("Version not found")

    debug = f"""Python: {python_version}\n
Gtk: {gtk_version}\n
Jpegoptim: {jpegoptim}\n
Oxipng: {oxipng}\n
pngquant: {pngquant}\n
Libwebp: {libwebp}\n
Scour: {scour}\n"""

    return debug


def extract_version(text):
    # regular expression to match the version string,
    # consists of three groups of one or more digits separated by dots
    version_regex = r"(\d+\.\d+\.\d+)"

    match = re.search(version_regex, text)
    if match:
        version_string = match.group(
            1
        )  # extract the version string from the match object
        return version_string
    else:
        return _("Version not found")
